#include "osdep.h"
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>

#include <fcntl.h>

#include <Strings.h>
#include <setjmp.h>

#include "helpers.h"
#include "pathname.h"
#include "revision.h"
#include "zip.h" 
#include "recurse.h"


int error_level;   /* used only in ziperr() */
extern const char ResourceMark[255]; /* var is initialized in file helpers.c */


/* #define USE_SIOUX    Comment out if extra Console-Code is used */      

const char MacPathEnd = ':';   /* the macintosh dir seperator */

extern jmp_buf EnvForExit;
MacZipGlobals	MacZip;

int add_UT_ef( struct zlist far *z );
int add_JLEE_ef( struct zlist far *z );  /* old mac extra field */
void make_extrafield_JLEE(char *l_ef);
int DoCurrentDir(void);

void DoAboutBox(void);
void DoQuit(void);
void DoEventLoop(void);

void ZipInitAllVars(void);


#ifdef USE_SIOUX
#include <sioux.h>
void DoWarnUserDupVol( char *FullPath );


void printerr(const char *msg,int cond,int err,int line,char *file,const char *msg2)
{

if (cond != 0)
	{
	fprintf(stderr,"\nint err: %d: %s %d [%d/%s] %s",clock(),msg,err,line,file,msg2);
	}
}


/* 
Program execution starts here with Metrowerks SIOUX-Console */
int main(int argc,char **argv)
{
int return_code;

	SIOUXSettings.asktosaveonclose = FALSE;
	SIOUXSettings.showstatusline = TRUE;
	
	SIOUXSettings.columns = 100;
	SIOUXSettings.rows    = 40;
	
	/* 0x01 = MacZip Johnny Lee's; 0x00 = new (my) MacZip */
MacZip.MacZipMode = NewZipMode;

argc = ccommand(&argv);
PrintArguments(argc,argv);

ZipInitAllVars();

return_code = zipmain(argc,argv);

printf("\n\n Finish");
return return_code;
}



void DoWarnUserDupVol( char *FullPath )
{
char VolName[257];
GetVolumeFromPath(FullPath,  VolName);

 printf("\n There are more than one volume that has the same name !!");
 printf("\n ");
 printf("\n Volume: %s",VolName);
 printf("\n ");
 printf("\n This port has one weak point: It is based on pathnames !!");
 printf("\n The use of pathnames, however, is highly discouraged.");
 printf("\n Avoid mounting Volumes with the same name !!");

}

#endif



/******************************/
/*  Function version_local()  */
/******************************/

void version_local()
{
/* prints e.g: 
Compiled with  Metrowerks CodeWarrior version 2000 for  PowerPC Processor 
 compile time:  Feb  4 1998 17:49:49.
*/
 
static ZCONST char CompiledWith[] = "\n\nCompiled with %s %x for %s \n %s %s %s.\n\n";

    printf(CompiledWith,


#ifdef __MWERKS__
      " Metrowerks CodeWarrior version", __MWERKS__,
#endif


#ifdef __MC68K__
      " MC68K Processor", 
#else
      " PowerPC Processor", 
#endif

#ifdef __DATE__
      "compile time: ", __DATE__, __TIME__
#else
      "", "", ""
#endif
    );
} /* end function version_local() */



int deletedir(char *path)
{
OSErr	err;
FSSpec	Spec;
short fullPathLength = strlen(path);

err = FSpLocationFromFullPath(fullPathLength,path, &Spec);
printerr("FSpLocationFromFullPath:",err,err,__LINE__,__FILE__,path);

err = FSpDelete(&Spec);
printerr("FSpDelete:",err,err,__LINE__,__FILE__,path);

return err;
}



void setfiletype(char *new_f, unsigned long Creator, unsigned long Type)
{
OSErr	err;

if (strcmp(zipfile,new_f) == 0) err = FSpChangeCreatorType(&MacZip.ZipFileSpec,Creator,Type);
printerr("FSpChangeCreatorType:",err,err,__LINE__,__FILE__,new_f);

return;
}



/* Extra field info:
   - 'UT' - UNIX time extra field

   This is done the same way ../unix/unix.c stores the 'UT'/'Ux' fields
   (full data in local header, only modification time in central header),
   with the 'Mc' field added to the end and the size of the 'Mc' field
   in the central header.
 */
int set_extra_field(struct zlist far *z, iztimes *z_utim)
  /* store full data in local header but just modification time stamp info
     in central header */
{
    int retval;


    /* Tell picky compilers to shut up about unused variables. */
    z_utim = z_utim;

    /* Check to make sure z is valid. */
    if( z == NULL ) {
        return ZE_LOGIC;
    }
    
/*   PrintFileInfo(); */
    
/* Johnny Lee's extra field implementation does not support 'UT' - UNIX time extra field */
/*
if (MacZip.MacZipMode != JohnnyLee)
	{
    retval = add_UT_ef( z );
    if( retval != ZE_OK ) return retval;
    }
    */

    retval = add_JLEE_ef( z );
    if( retval != ZE_OK ) {
        return retval;
    }

MacZip.isMacStatValid = false;		

    return ZE_OK;
}




/* ---------------------------------------------------------------------- */
/* Add a 'UT' extra field to the zlist data pointed to by z.              */

#define EB_L_UT_SIZE    (EB_HEADSIZE + EB_UT_LEN(2))
#define EB_C_UT_SIZE    (EB_HEADSIZE + EB_UT_LEN(1))

int add_UT_ef( struct zlist far *z )
{
    char        *l_ef = NULL;
    char        *c_ef = NULL;
    struct stat  s;

    /* We can't work if there's no entry to work on. */
    if( z == NULL ) {
        return ZE_LOGIC;
    }

    /* Check to make sure we've got enough room in the extra fields. */
    if( z->ext + EB_L_UT_SIZE > EXTRAFLD_MAX ||
        z->cext + EB_C_UT_SIZE > EXTRAFLD_MAX ) {
        return ZE_MEM;
    }

    /* stat() the file (or the symlink) to get the data; if we can't get */
    /* the data, there's no point in trying to fill out the fields.      */
    if(LSSTAT( z->name, &s ) ) {
           return ZE_OPEN;
    }

    /* Allocate memory for the local and central extra fields. */
    if( z->extra && z->ext != 0 ) {
        z->extra = (char *)realloc( z->extra, z->ext + EB_L_UT_SIZE );
        l_ef = z->extra + z->ext;
    } else {
        z->extra = (char *)malloc( EB_L_UT_SIZE );
        z->ext = 0;
        l_ef = z->extra;
    }
    if( l_ef == NULL ) {
        return ZE_MEM;
    }

    if( z->cextra && z->cext != 0 ) {
        z->cextra = (char *)realloc( z->cextra, z->cext + EB_C_UT_SIZE );
        c_ef = z->cextra + z->cext;
    } else {
        z->cextra = (char *)malloc( EB_C_UT_SIZE );
        z->cext = 0;
        c_ef = z->cextra;
    }
    if( c_ef == NULL ) {
        return ZE_MEM;
    }

    /* Now add the local version of the field. */
    *l_ef++ = 'U';
    *l_ef++ = 'T';
    *l_ef++ = (char)(EB_UT_LEN(2)); /* length of data in local EF */
    *l_ef++ = (char)(EB_UT_LEN(2) >> 8);
    *l_ef++ = (char)(EB_UT_FL_MTIME | EB_UT_FL_ATIME);
    *l_ef++ = (char)(s.st_mtime);
    *l_ef++ = (char)(s.st_mtime >> 8);
    *l_ef++ = (char)(s.st_mtime >> 16);
    *l_ef++ = (char)(s.st_mtime >> 24);
    *l_ef++ = (char)(s.st_atime);
    *l_ef++ = (char)(s.st_atime >> 8);
    *l_ef++ = (char)(s.st_atime >> 16);
    *l_ef++ = (char)(s.st_atime >> 24);

    z->ext += EB_L_UT_SIZE;

    /* Now add the central version. */
    *c_ef++ = 'U';
    *c_ef++ = 'T';
    *c_ef++ = (char)(EB_UT_LEN(1)); /* length of data in central EF */
    *c_ef++ = (char)(EB_UT_LEN(1) >> 8);
    *c_ef++ = (char)(EB_UT_FL_MTIME);
    *c_ef++ = (char)(s.st_mtime);
    *c_ef++ = (char)(s.st_mtime >> 8);
    *c_ef++ = (char)(s.st_mtime >> 16);
    *c_ef++ = (char)(s.st_mtime >> 24);

    z->cext += EB_C_UT_SIZE;

    return ZE_OK;
}




/* ---------------------------------------------------------------------- */
/* Add a 'JLEE' extra field to the zlist data pointed to by z.            */
/* This is the (old) Info-zip resource-fork extra block for Macintosh
   (last Revision 1996-09-22)  Layout made by Johnny Lee, Code made by me :-)  */
#define EB_L_JLEE_LEN  64           /* fixed length of JLEE's header */
#define EB_C_JLEE_LEN  64           /* fixed length of JLEE's header same as local header */

#define EB_L_JLEE_SIZE    (EB_HEADSIZE + EB_L_JLEE_LEN) 
#define EB_C_JLEE_SIZE    (EB_HEADSIZE + EB_C_JLEE_LEN)

int add_JLEE_ef( struct zlist far *z )
{
    char *l_ef       = NULL;
    char *c_ef       = NULL;
     

    /* Check to make sure we've got enough room in the extra fields. */
    if( z->ext + EB_L_JLEE_SIZE > EXTRAFLD_MAX ||
        z->cext + EB_C_JLEE_SIZE > EXTRAFLD_MAX ) {
        return ZE_MEM;
    }


    /* Allocate memory for the local extra fields. */
    if( z->extra && z->ext != 0 ) {
        z->extra = (char *)realloc( z->extra, z->ext + EB_L_JLEE_SIZE );
        l_ef = z->extra + z->ext;
    } else {
        z->extra = (char *)malloc( EB_L_JLEE_SIZE );
        z->ext = 0;
        l_ef = z->extra;
    }
    if( l_ef == NULL ) {
        return ZE_MEM;
    }

    /* Allocate memory for the  central extra fields. */
    if( z->cextra && z->cext != 0 ) {
        z->cextra = (char *)realloc( z->cextra, z->cext + EB_C_JLEE_SIZE );
        c_ef = z->cextra + z->cext;
    } else {
        z->cextra = (char *)malloc( EB_C_JLEE_SIZE );
        z->cext = 0;
        c_ef = z->cextra;
    }
    if( c_ef == NULL ) {
        return ZE_MEM;
    }
  
  
    if( verbose ) {
        print_extra_info();
    }
    
   
    /**
    **
    **  Now add the local version of the field. 
    **/  
	make_extrafield_JLEE(l_ef);
    z->ext += EB_L_JLEE_SIZE;		
   
   
    /**
    **
    **  Now add the central version of the field. 
    **  It's identical to the local header. I wonder why ??
    *  the first two fields are in Intel little-endian format  */ 
	make_extrafield_JLEE(c_ef);
    z->cext += EB_C_JLEE_SIZE;											   

    return ZE_OK;
}



/*
* This is a implementation of Johnny Lee's extra field.
* I never saw Johnny Lee's code. My code is based on the extra-field definition
* mac (see latest appnote 1997-03-11)
* and on some experiments with Johnny Lee's Zip-app version 1.0, 1994
*
* Unfortunately I can not agree with his extra-field layout.
*   - it wasted space
*   - and holds not all mac-specific information 
* 
* I coded this extra-field only for testing purposes.
* I don't want support this extra-field. Please use my implementation.
*/
void make_extrafield_JLEE(char *ef)
{
    unsigned char LengthVolumeName;
    static char	VolumeName[23];  /* old limitation; don't change the size */
	
	if (MacZip.isMacStatValid == false)
		{
		fprintf(stderr,"Internal Logic Error: [%d/%s] MacStat is out of sync !",__LINE__,__FILE__);
		exit(-1);
		}
	
	LengthVolumeName = GetVolumeFromPath(MacZip.FullPath, VolumeName);


    /*  the first two fields are in Intel little-endian format  */ 
    *ef++ = 0xC8;									   /* tag for this extra block */							
    *ef++ = 0x07;											
    
    *ef++ = (char)(EB_L_JLEE_LEN);		               /* total data size this block */		
    *ef++ = (char)((EB_L_JLEE_LEN) >> 8);		
    
    /*  the following fields are in motorola big-endian format  */ 
    *ef++ = 'J';									   /* extra field signature:       4 Bytes */
    *ef++ = 'L';			
    *ef++ = 'E';		
    *ef++ = 'E';		
    
  				/* Start Macintosh Finder FInfo structure   16 Bytes overall */
    												   /* Type:                        4 Bytes */	
    *ef++ = (char)((unsigned long)MacZip.fpb.hFileInfo.ioFlFndrInfo.fdType >> 24);		
    *ef++ = (char)((unsigned long)MacZip.fpb.hFileInfo.ioFlFndrInfo.fdType >> 16);	
    *ef++ = (char)((unsigned long)MacZip.fpb.hFileInfo.ioFlFndrInfo.fdType >> 8);	
    *ef++ = (char)((unsigned long)MacZip.fpb.hFileInfo.ioFlFndrInfo.fdType);				
    
    												   /* Creator:                     4 Bytes */	
    *ef++ = (char)((unsigned long)MacZip.fpb.hFileInfo.ioFlFndrInfo.fdCreator >> 24);		
    *ef++ = (char)((unsigned long)MacZip.fpb.hFileInfo.ioFlFndrInfo.fdCreator >> 16);	
    *ef++ = (char)((unsigned long)MacZip.fpb.hFileInfo.ioFlFndrInfo.fdCreator >> 8);	
    *ef++ = (char)((unsigned long)MacZip.fpb.hFileInfo.ioFlFndrInfo.fdCreator);		


if (!MacZip.isDirectory)
{
     										     /* file Finder Flags:                2 Bytes */	
    *ef++ = (char)((unsigned short)MacZip.fpb.hFileInfo.ioFlFndrInfo.fdFlags >> 8);	
    *ef++ = (char)((unsigned short)MacZip.fpb.hFileInfo.ioFlFndrInfo.fdFlags);		

  				                    /* Finders Icon position of a file*/
     												   /* V/Y-Position:                2 Bytes */	
    *ef++ = (char)((unsigned short)MacZip.fpb.hFileInfo.ioFlFndrInfo.fdLocation.v >> 8);		
    *ef++ = (char)((unsigned short)MacZip.fpb.hFileInfo.ioFlFndrInfo.fdLocation.v);		
     												   /* H/X-Position:                2 Bytes */	
    *ef++ = (char)((unsigned short)MacZip.fpb.hFileInfo.ioFlFndrInfo.fdLocation.h >> 8);		
    *ef++ = (char)((unsigned short)MacZip.fpb.hFileInfo.ioFlFndrInfo.fdLocation.h);		
}
else
{
     								         /* directory Finder Flags:                2 Bytes */	
    *ef++ = (char)((unsigned short)MacZip.fpb.dirInfo.ioDrUsrWds.frFlags >> 8);	
    *ef++ = (char)((unsigned short)MacZip.fpb.dirInfo.ioDrUsrWds.frFlags);		

  				                    /* Finders Icon position of a directory */
     												   /* V/Y-Position:                2 Bytes */	
    *ef++ = (char)((unsigned short)MacZip.fpb.dirInfo.ioDrUsrWds.frLocation.v >> 8);		
    *ef++ = (char)((unsigned short)MacZip.fpb.dirInfo.ioDrUsrWds.frLocation.v);		
     												   /* H/X-Position:                2 Bytes */	
    *ef++ = (char)((unsigned short)MacZip.fpb.dirInfo.ioDrUsrWds.frLocation.h >> 8);		
    *ef++ = (char)((unsigned short)MacZip.fpb.dirInfo.ioDrUsrWds.frLocation.h);		
}

     										    /* fdFldr Folder containing file ??    2 Bytes */
    *ef++ = 0x00;									   /* seems to be always zero */
    *ef++ = 0x00;	
  				/* End Macintosh Finder FInfo structure */
	

    												   /* File Creation Date           4 Bytes */
    *ef++ = (char)((unsigned long)MacZip.CreatDate >> 24);		
    *ef++ = (char)((unsigned long)MacZip.CreatDate >> 16);	
    *ef++ = (char)((unsigned long)MacZip.CreatDate >> 8);	
    *ef++ = (char)((unsigned long)MacZip.CreatDate);				

    												   /* File Modification Date       4 Bytes */
    *ef++ = (char)((unsigned long)MacZip.ModDate >> 24);		
    *ef++ = (char)((unsigned long)MacZip.ModDate >> 16);	
    *ef++ = (char)((unsigned long)MacZip.ModDate >> 8);	
    *ef++ = (char)((unsigned long)MacZip.ModDate);				

    												   /* info Bits                    4 Bytes */
    *ef++ = 0x00;		
    *ef++ = 0x00;	
    *ef++ = 0x00;	
    *ef++ = (char) (MacZip.CurrentFork == DataFork);   /* 0x01 = data-fork; 0x00 = resource-fork */
        
        												 /* file's location folder ID  4 Bytes */
    *ef++ = (char)((unsigned long)MacZip.fpb.hFileInfo.ioFlParID >> 24);		
    *ef++ = (char)((unsigned long)MacZip.fpb.hFileInfo.ioFlParID >> 16);	
    *ef++ = (char)((unsigned long)MacZip.fpb.hFileInfo.ioFlParID >> 8);	
    *ef++ = (char)((unsigned long)MacZip.fpb.hFileInfo.ioFlParID);				
    

    												   /* length of volume-name        1 Byte */
    *ef++ = LengthVolumeName;		
    												   /* volume-name          fix to 27 Bytes */
    memcpy( ef, VolumeName, (size_t) 22 );
																               /* ============  */ 
									                                           /*     64 Bytes  */							                                           									       									   
}





char *ex2in(char *externalFilen, int isdir,int *pdosflag)
/* char *externalFilen     external file name */
/* int isdir               input: externalFilen is a directory */
/* int *pdosflag           output: force MSDOS file attributes? */
/* Convert the external file name to a zip file name, returning the malloc'ed
   string or NULL if not enough memory. */
{
  char *internalFilen;              /* internal file name (malloc'ed) */
  char *t;              /* shortened name */
  char *tmpPtr;
  char *Pathname;
  char buffer[NAME_MAX];
  int dosflag;
  
  dosflag = dosify;  /* default for non-DOS and non-OS/2 */
  	
  /* Find starting point in name before doing malloc */
  for (t = externalFilen; *t == MacPathEnd; t++)
      ;  
  
  if (!MacZip.StoreFullPath)
  		{
  		Pathname = StripPartialDir(buffer, MacZip.SearchDir,t);
  		}
  else
  		{
  		Pathname = t;
  		}

  /* Make changes, if any, to the copied name (leave original intact) */
  if (!pathput)
  	{
    t = last(Pathname, MacPathEnd);
    }
  else t = Pathname;


  /* Malloc space for internal name and copy it */
  if ((internalFilen = malloc(strlen(t) + 1 + strlen(ResourceMark) )) == NULL)
    return NULL;
    
  strcpy(internalFilen, t);
    
	/* first we have to eliminate illegal chars */
    /* unix path delimiter '/' is under mac HFS a legal char ! */
  for (tmpPtr = internalFilen; *tmpPtr; tmpPtr++)             /* exchange '/' in '_' */
    if (*tmpPtr == '/')
      *tmpPtr = '_';


	/* now we can convert HFS-Path in Unix-Path  */    
      /* Normalize path delimiter as '/'. */
  for (tmpPtr = internalFilen; *tmpPtr; tmpPtr++)             /* use / consistently */
    if (*tmpPtr == ':')
      *tmpPtr = '/';
    
  /* Returned malloc'ed name */
  if (pdosflag)
    *pdosflag = dosflag;

  if (isdir) 
  	{
  	return internalFilen;      /* avoid warning on unused variable */
  	}

  if (dosify)
  	{
    msname(internalFilen);
    printf("\n ex2in: %s",internalFilen);
    }

  return internalFilen;
}




int wild(char *Pathpat)
                /* path/pattern to match */
/* If not in exclude mode, expand the pattern based on the contents of the
   file system.  Return an error code in the ZE_ class. */
{
FSSpec Spec;
char fullpath[NAME_MAX];
OSErr	err;

/* for switch '-R' -> '.' means currend dir */
if (strcmp(Pathpat,".") == 0) strcpy(Pathpat,"*");

if (noisy) printf("\nCurrent Dir: [%s]\n",MacZip.CurrentPath);

strcpy(MacZip.Pattern,Pathpat); 
 
if (recurse) 
	{
	MacZip.SearchLevels = 0; /* if 0 we aren't checking levels */
	}
else 
	{
	MacZip.StoreFoldersAlso = false;
	MacZip.SearchLevels = 1;
	}

/* make complete path */
GetCompletePath(fullpath, MacZip.Pattern, &Spec,&err); 
err = PrintUserHFSerr((err != -43) && (err != 0), err, MacZip.Pattern); 
printerr("GetCompletePath:",err,err,__LINE__,__FILE__,fullpath); 

/* extract the filepattern */
GetFilename(MacZip.Pattern,fullpath);

/* extract Path and get FSSpec of search-path */
/* get FSSpec of search-path  ; we need a dir to start searching for filenames */
TruncFilename(MacZip.SearchDir,fullpath,&Spec,&err);
printerr("TruncFilename:",err,err,__LINE__,__FILE__,fullpath);

if ((MacZip.SearchLevels == 0) && (noisy == true))
	{
	printf("\nSearch Pattern: [%s]   Levels: all",MacZip.Pattern);
	}
else
	{
	printf("\nSearch Pattern: [%s]   Levels: %d",MacZip.Pattern,MacZip.SearchLevels);
	}

if (noisy) printf("\nSearch Path:    [%s]\n",MacZip.SearchDir);

/* we are working only with pathnames this can cause big problems on a mac ...*/
if (CheckMountedVolumes(MacZip.SearchDir) > 1)
	DoWarnUserDupVol(MacZip.SearchDir);

/* start getting all filenames */
err = FSpRecurseDirectory(&Spec, MacZip.SearchLevels);
printerr("FSpRecurseDirectory:",err,err,__LINE__,__FILE__,"");

return ZE_OK;
}



char *in2ex(char *n)    /* internal file name */
/* Convert the zip file name to an external file name, returning the malloc'ed
   string or NULL if not enough memory. */
{
  char *x;              /* external file name */

  if ((x = malloc(strlen(n) + 1)) == NULL)
    return NULL;
 
 RfDfFilen2Real(x,n);
 
 return x;
}



int procname(char *filename)           /* name to process */
/* Process a name .  Return
   an error code in the ZE_ class. */
{
  int rc;                /* matched flag */

    /* add or remove name of file */
    
rc = newname(filename, MacZip.isDirectory); 

return rc;
}


ulg filetime(char *f, ulg *a, long *n,iztimes *t)
/* char *f;                 name of file to get info on */
/* ulg *a;                  return value: file attributes */
/* long *n;                 return value: file size */
/* iztimes *t;              return value: access, modific. and creation times */
/* If file *f does not exist, return 0.  Else, return the file's last
   modified date and time as an MSDOS date and time.  The date and
   time is returned in a long with the date most significant to allow
   unsigned integer comparison of absolute times.  Also, if a is not
   a NULL pointer, store the file attributes there, with the high two
   bytes being the Unix attributes, and the low byte being a mapping
   of that to DOS attributes.  If n is not NULL, store the file size
   there.  If t is not NULL, the file's access, modification and creation
   times are stored there as UNIX time_t values.
   If f is "-", use standard input as the file. If f is a device, return
   a file size of -1 */
{
  struct stat s;        /* results of stat() */
  char name[NAME_MAX];
  int len = strlen(f);

 strcpy(name, f);

 if (len == 0) return 0;
 *a = 0;
 
  if (MacStat(name, &s) != 0)
    /* Accept about any file kind including directories
     * (stored with trailing : with -r option)
     */
    return 0;

  if (a != NULL) {
    *a = ((ulg)s.st_mode << 16) | !(s.st_mode & S_IWRITE);
    if (MacZip.isDirectory) { 
      *a = MSDOS_DIR_ATTR; 
    }
  }
  if (n != NULL)
    *n = (s.st_mode & S_IFMT) == S_IFREG ? s.st_size : -1L;
  if (t != NULL) {
    t->atime = s.st_atime;
    t->mtime = s.st_mtime;
    t->ctime = t->mtime;   /* best guess, (s.st_ctime: last status change!!) */
  }
  return unix2dostime(&s.st_mtime);
}



void stamp(char *f, ulg d)
/* char *f;                name of file to change */
/* ulg d;                  dos-style time to change it to */
/* Set last updated and accessed time of file f to the DOS time d. */
{
  time_t u[2];          /* argument for utime() */

f = f;

  /* Convert DOS time to time_t format in u */

  u[0] = u[1] = dos2unixtime(d);
/*  utime(f, u);  */
}




void ZipInitAllVars(void)
{

getcwd(MacZip.CurrentPath, sizeof(MacZip.CurrentPath));
MacZip.MacZipMode = JohnnyLee;  
/*  MacZip.MacZipMode = NewZipMode;   */
 
MacZip.DataForkOnly = false;
MacZip.CurrentFork = NoFork;

MacZip.StoreFoldersAlso = true;

MacZip.FoundFiles = 0;
MacZip.FoundDirectories = 0;
MacZip.BytesOfData = 0;

MacZip.StoreFullPath = false;
MacZip.StatingProgress = false;
MacZip.IncludeInvisible = false;

MacZip.isMacStatValid = false;

MacZip.IgnoreGMToffset = true;

error_level = 0;

}


char *GetComment(char *filename)
{
unsigned char comment[257];
OSErr 		err;
static char	buffer[NAME_MAX];
char *tmpPtr;
short fullPathLength;

	/* now we can convert Unix-Path in HFS-Path */    
for (tmpPtr = filename; *tmpPtr; tmpPtr++)             
    if (*tmpPtr == '/')
      *tmpPtr = ':';

/* make fullpath and get FSSpec */
strcpy(buffer,MacZip.SearchDir);
strcat(buffer,filename);
fullPathLength = strlen(buffer);

err = FSpLocationFromFullPath(fullPathLength,buffer, &MacZip.fileSpec);
printerr("FSpLocationFromFullPath:",err,err,__LINE__,__FILE__,buffer); 

err = FSpDTGetComment(&MacZip.fileSpec, comment);
printerr("FSpDTGetComment:",(err != -5012) && (err != 0),err,__LINE__,__FILE__,filename); 
PToCCpy(comment,buffer);

if (noisy) printf("\n%20s -> %s",filename, buffer);
return buffer;
}




void PrintStatProgress(char *msg)
{

if (!noisy) return;     /* do no output if noisy is false */

MacZip.StatingProgress = true;

if (strcmp(msg,"done") == 0) 
	{
	MacZip.StatingProgress = false;
	printf("\n ... done \n\n");
	}
else printf("\n %s",msg);

}


#ifndef USE_SIOUX

char *GetZipVersionsInfo(void)
{
static char ZipVersion[100];

sprintf(ZipVersion, "Zip Module\r%d.%d%d%s of %s", Z_MAJORVER, Z_MINORVER,
        Z_PATCHLEVEL, Z_BETALEVEL, REVDATE);

return ZipVersion;
}



char *GetZipCopyright(void)
{
static char CopyR[300];

strcpy(CopyR, copyright[0]);
strcat(CopyR, copyright[1]);
strcat(CopyR, "\r\rPlease send bug reports to the authors at\rZip-Bugs@lists.wku.edu");

return CopyR;
}



char *GetZipVersionLocal(void)
{
static char ZipVersionLocal[50];

sprintf(ZipVersionLocal, "[%s %s]", __DATE__, __TIME__);
    
return ZipVersionLocal;
} 



#endif  /*   #ifndef USE_SIOUX  */


