/*** unixlike.c; stuff only required for the Mac port ***/

/* 

Many functions of this file are from:

"Tcl 8.0p1 for Macintosh

 by Ray Johnson
 Sun Microsystems Laboratories
 rjohnson@eng.sun.com"

*/

#include <string.h>
#include <stdio.h>
#include <unistd.h>

#include "osdep.h"
#include "zip.h"
#include "unixlike.h"
#include "helpers.h"
#include "pathname.h"
#include "macstuff.h"
#include "macglob.h"

extern MacZipGlobals	MacZip;
extern int errno; 

/*
 * Static variables used by the stat function.
 */
static int initalized = false;
static int indx=0;

/* offset to current entry, >0 is open, 0 is closed, -1 is at end */
static CInfoPBRec finfo;
static unsigned char fname[257];

/*
 *----------------------------------------------------------------------
 *
 * MacStat --
 *
 *	This function replaces the library version of stat.  The stat
 *	function provided by most Mac compiliers is rather broken and
 *	incomplete.
 *
 * Results:
 *	See stat documentation.
 *
 * Side effects:
 *	See stat documentation.
 *
 *----------------------------------------------------------------------
 */

int MacStat(char *Fname, struct stat *buf)
{
    OSErr 			err, rc;
    short 			fullPathLength;
    Handle 			hFullPath;
    char   			path[NAME_MAX], path2[NAME_MAX];
    struct tm 		*tp;
    HVolumeParam 	vpb;
   
	memset(buf,0,sizeof(buf));   /* zero out all fields */
	
	RfDfFilen2Real(path2,Fname);
	
	GetCompletePath(path, path2, &MacZip.fileSpec, &err);
    err = PrintUserHFSerr((err != -43) && (err != 0), err, path);
    printerr("GetCompletePath:",err,err,__LINE__,__FILE__,path); 

    if (err != noErr) {
	errno = err;
	return -1;
    }
  
    
    /*  Collect here some more information, it's not related to Macstat. 
        (note: filespec gets changed later in this function) */
    memset(MacZip.FullPath, 0x00, sizeof(MacZip.FullPath)); /* clear string-buffer */
	rc = FSpGetFullPath(&MacZip.fileSpec, &fullPathLength, &hFullPath);
	strncpy(MacZip.FullPath,*hFullPath,fullPathLength);
	DisposeHandle(hFullPath);   /* we don't need it any more */
	/*  Collect some more information not related to Macstat */
    
    
    /*
     * Fill the fpb & vpb struct up with info about file or directory.
     */
     
    FSpGetDirectoryID(&MacZip.fileSpec, &MacZip.dirID, &MacZip.isDirectory);
    vpb.ioVRefNum = MacZip.fpb.hFileInfo.ioVRefNum =  MacZip.fileSpec.vRefNum;
    vpb.ioNamePtr = MacZip.fpb.hFileInfo.ioNamePtr =  MacZip.fileSpec.name;
    
    if (MacZip.isDirectory) {
	MacZip.fpb.hFileInfo.ioDirID =  MacZip.fileSpec.parID;
    } else {
	MacZip.fpb.hFileInfo.ioDirID = MacZip.dirID;
    }

    MacZip.fpb.hFileInfo.ioFDirIndex = 0;
    err = PBGetCatInfoSync((CInfoPBPtr)&MacZip.fpb);
    
    if (err == noErr) {
	vpb.ioVolIndex = 0;
	err = PBHGetVInfoSync((HParmBlkPtr)&vpb);
	if (err == noErr && buf != NULL) {
	    /* 
	     * Files are always readable by everyone.
	     */
	     
	    buf->st_mode = S_IRUSR | S_IRGRP | S_IROTH;

	    /* 
	     * Use the Volume Info & File Info to fill out stat buf.
	     */
	    if (MacZip.fpb.hFileInfo.ioFlAttrib & 0x10) {
			buf->st_mode |= S_IFDIR;
			buf->st_nlink = 2; 
	    } else {
			buf->st_nlink = 1;
		if (MacZip.fpb.hFileInfo.ioFlFndrInfo.fdFlags & 0x8000) {
		    buf->st_mode |= S_IFLNK; 

		} else {
		    buf->st_mode |= S_IFREG;
		}
	    }
	   
   
	    if ((MacZip.fpb.hFileInfo.ioFlAttrib & 0x10) || (MacZip.fpb.hFileInfo.ioFlFndrInfo.fdType == 'APPL')) {
	    	/*
	    	 * Directories and applications are executable by everyone.
	    	 */
   	    	if (verbose) 
				{
				if (MacZip.StatingProgress)
					printf(".");
				}
	       
	    	buf->st_mode |= S_IXUSR | S_IXGRP | S_IXOTH;
	    	}
	    if ((MacZip.fpb.hFileInfo.ioFlAttrib & 0x01) == 0){
	    	/* 
	    	 * If not locked, then everyone has write acces.
	    	 */
	    	 
	        buf->st_mode |= S_IWUSR | S_IWGRP | S_IWOTH;
	        }
	    buf->st_ino = MacZip.fpb.hFileInfo.ioDirID;
	    buf->st_dev = MacZip.fpb.hFileInfo.ioVRefNum;
	    buf->st_uid = -1;
	    buf->st_gid = -1;
	    buf->st_rdev = 0;
	    	    				
	    if (MacZip.CurrentFork == ResourceFork) buf->st_size = MacZip.fpb.hFileInfo.ioFlRLgLen;
	    else buf->st_size = MacZip.fpb.hFileInfo.ioFlLgLen;
	    
	    buf->st_blksize = vpb.ioVAlBlkSiz;
	    buf->st_blocks = (buf->st_size + buf->st_blksize - 1)
		/ buf->st_blksize;

	    /*
	     * The times returned by the Mac file system are in the
	     * local time zone.  We convert them to GMT so that the
	     * epoch starts from GMT.  This is also consistant with
	     * what is returned from "clock seconds".
	     */
	    if (initalized == false) {
		MachineLocation loc;
		
		tp->tm_sec  = 0;
		tp->tm_min  = 0;		
		tp->tm_hour = 0;
		tp->tm_mon  = 0;
		tp->tm_year = 4;
		tp->tm_mday = 1;

		MacZip.tm_CorrectOffset = mktime(tp); /* strange: for some reason I got an offset
										            of one year and one day */
		ReadLocation(&loc);
		
		MacZip.gmt_offset = loc.u.gmtDelta & 0x00ffffff;
		if (MacZip.gmt_offset & 0x00800000) {
		    MacZip.gmt_offset = MacZip.gmt_offset | 0xff000000;
		}
		initalized = true;
	    }
	   
	 if (!MacZip.isDirectory)
		{
  		MacZip.CreatDate  = MacZip.fpb.hFileInfo.ioFlCrDat;
		MacZip.ModDate    = MacZip.fpb.hFileInfo.ioFlMdDat + MacZip.tm_CorrectOffset;
		MacZip.BackDate   = MacZip.fpb.hFileInfo.ioFlBkDat + MacZip.tm_CorrectOffset;
		}
	else
		{
  		MacZip.CreatDate  = MacZip.fpb.dirInfo.ioDrCrDat;
		MacZip.ModDate    = MacZip.fpb.dirInfo.ioDrMdDat + MacZip.tm_CorrectOffset;
		MacZip.BackDate   = MacZip.fpb.dirInfo.ioDrBkDat + MacZip.tm_CorrectOffset;
	} 

if (MacZip.IgnoreGMToffset == false)
	{
	MacZip.CreatDate  = MacZip.CreatDate - MacZip.gmt_offset;
	MacZip.ModDate    = MacZip.ModDate   - MacZip.gmt_offset;
	MacZip.BackDate   = MacZip.BackDate  - MacZip.gmt_offset;
	}
		
	    buf->st_atime = buf->st_mtime = MacZip.ModDate;
	    buf->st_ctime = MacZip.CreatDate;
	}
    }


    if (err != noErr) {
	errno = err;
    }    
    
    MacZip.isMacStatValid = true;
    
    return (err == noErr ? 0 : -1);
}





/*
 *----------------------------------------------------------------------
 *
 * chmod --
 *
 * Results:
 *	See chmod documentation.
 *
 * Side effects:
 *	See chmod documentation.
 *
 *----------------------------------------------------------------------
 */
 
int chmod(char *path, int mode)
{
    HParamBlockRec hpb;
    OSErr err;
      
    hpb.fileParam.ioNamePtr = c2pstr(path);
    hpb.fileParam.ioVRefNum = 0;
    hpb.fileParam.ioDirID = 0;
    
    if (mode & 0200) {
        err = PBHRstFLockSync(&hpb);
    } else {
        err = PBHSetFLockSync(&hpb);
    }

    if (err != noErr) {
        errno = err;
        return -1;
    }
    
    return 0;
}



/*
 *----------------------------------------------------------------------
 *
 * readlink --
 *
 *	This function replaces the library version of readlink.
 *
 * Results:
 *	See readlink documentation.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

int readlink(char *path, char *buf, int size)
{
    HFileInfo fpb;
    OSErr err;
    FSSpec fileSpec;
    Boolean isDirectory;
    Boolean wasAlias;
    long dirID;
    char fileName[NAME_MAX];
    char *end;
    Handle theString = NULL;
    int pathSize, fullPathLength;

    RfDfFilen2Real(path,path);
   
    /*
     * Remove ending colons if they exist.
     */
    while ((strlen(path) != 0) && (path[strlen(path) - 1] == ':')) {
	path[strlen(path) - 1] = NULL;
    }

    if (strchr(path, ':') == NULL) {
	strcpy(fileName, path);
	path = NULL;
    } else {
	end = strrchr(path, ':') + 1;
	strcpy(fileName, end);
	*end = NULL;
    }
    c2pstr(fileName);
    
    /*
     * Create the file spec for the directory of the file
     * we want to look at.
     */
    if (path != NULL) {
		fullPathLength = strlen(path);
		err = FSpLocationFromFullPath(fullPathLength,path, &fileSpec);
    	printerr("FSpLocationFromFullPath:",err,err,__LINE__,__FILE__,path); 
	if (err != noErr) {
	    errno = EINVAL;
	    return -1;
	}
    } else {
	FSMakeFSSpecCompat(0, 0, NULL, &fileSpec);
    }
    
    /*
     * Fill the fpb struct up with info about file or directory.
     */
    FSpGetDirectoryID(&fileSpec, &dirID, &isDirectory);
    fpb.ioVRefNum = fileSpec.vRefNum;
    fpb.ioDirID = dirID;
    fpb.ioNamePtr = (StringPtr) fileName;

    fpb.ioFDirIndex = 0;
    err = PBGetCatInfoSync((CInfoPBPtr)&fpb);
    if (err != noErr) {
	errno = err;
	return -1;
    } else {
	if (fpb.ioFlAttrib & 0x10) {
	    errno = EINVAL;
	    return -1;
	} else {
	    if (fpb.ioFlFndrInfo.fdFlags & 0x8000) {
		/*
		 * The file is a link!
		 */
	    } else {
		errno = EINVAL;
		return -1;
	    }
	}
    }
    
    /*
     * If we are here it's really a link - now find out
     * where it points to.
     */
    err = FSMakeFSSpecCompat(fileSpec.vRefNum, dirID, (StringPtr) fileName, &fileSpec);
    if (err == noErr) {
	err = ResolveAliasFile(&fileSpec, true, &isDirectory, &wasAlias);
    }
    if ((err == fnfErr) || wasAlias) {
	err = FSpPathFromLocation(&fileSpec, &pathSize, &theString);
	if ((err != noErr) || (pathSize > size)) {
	    DisposeHandle(theString);
	    errno = ENAMETOOLONG;
	    return -1;
	}
    } else {
    	errno = EINVAL;
	return -1;
    }

    strncpy(buf, *theString, pathSize);
    DisposeHandle(theString);
    
    return pathSize;
}







